/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.h
  * @brief          : Header for main.c file.
  *                   This file contains the common defines of the application.
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2022 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __MAIN_H
#define __MAIN_H

#ifdef __cplusplus
extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "stm32f7xx_hal.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */

/* USER CODE END Includes */

/* Exported types ------------------------------------------------------------*/
/* USER CODE BEGIN ET */
/*
	Task state structure.
	In our simple scheduler, the task is determined only by its stack pointer and
	the start address (i.e. pointer to a function that implements the task)
	but in general, the structure should contain a task's state variable (flags)
	(e.g. RUNNING, STOPED, ACTIVE, ....)

	Pa3cio Bulic, 26.10.2021
*/
typedef struct {
	void* sp;    	        // task's stack pointer
	void (*pTask)(void); 	// start address of the task
	// int flags;
} task_table_t;


/*
	Hardware Stack Frame structure.

	Pa3cio Bulic, 26.10.2021
*/
typedef struct {
	uint32_t r0;
	uint32_t r1;
	uint32_t r2;
	uint32_t r3;
	uint32_t r12;
	uint32_t lr;
	uint32_t pc;
	uint32_t xpsr;
} hw_stack_frame_t;


/*
	Hardware Stack Frame structure.

	Pa3cio Bulic, 26.10.2021
*/
typedef struct {
	uint32_t r4;
	uint32_t r5;
	uint32_t r6;
	uint32_t r7;
	uint32_t r8;
	uint32_t r9;
	uint32_t r10;
	uint32_t r11;
} sw_stack_frame_t;
/* USER CODE END ET */

/* Exported constants --------------------------------------------------------*/
/* USER CODE BEGIN EC */
#define MAX_TASKS 4 			// število opravil
#define TASKS_STACK_SIZE 256    // velikost skladu za eno opravilo merjena v 4B
/* USER CODE END EC */

/* Exported macro ------------------------------------------------------------*/
/* USER CODE BEGIN EM */

/* USER CODE END EM */

/* Exported functions prototypes ---------------------------------------------*/
void Error_Handler(void);

/* USER CODE BEGIN EFP */
void create_task (int task_id, void* task_stack_base_address, void (*pTask)(void));
void init_tasks (void);
void switch_context (void);

void Opravilo1 (void);
void Opravilo2 (void);
void Opravilo3 (void);
void Opravilo0 (void);
/* USER CODE END EFP */

/* Private defines -----------------------------------------------------------*/
#define PERIOD 3000
#define PRESCALER 16000
#define LED_RED_Pin GPIO_PIN_13
#define LED_RED_GPIO_Port GPIOJ
#define LED_GREEN_Pin GPIO_PIN_5
#define LED_GREEN_GPIO_Port GPIOJ
#define USR_BTN_Pin GPIO_PIN_0
#define USR_BTN_GPIO_Port GPIOA
#define USR_BTN_EXTI_IRQn EXTI0_IRQn
/* USER CODE BEGIN Private defines */


/* USER CODE END Private defines */

#ifdef __cplusplus
}
#endif

#endif /* __MAIN_H */
